==> backup-0.6.module.tgz: modules/backup/main.act.php <==
<?php

	/*
	*	Gekko - Open Source Web Development Framework
	*	------------------------------------------------------------------------
	*	Copyright (C) 2004-2006, J. Carlos Nieto <xiam@users.sourceforge.net>
	*	This program is Free Software.
	*
	*	@package	Gekko
	*	@license	http://www.gnu.org/copyleft/gpl.html GNU/GPL License 2.0
	*	@author		J. Carlos Nieto <xiam@users.sourceforge.net>
	*	@version	CVS: $Id: main.act.php,v 1.1.1.1 2006/02/20 21:52:22 xiam Exp $
	*	@link		http://www.gekkoware.org
	*/

	function addsqlslashes($text) {
		$text = str_replace("\\", "\\\\", $text);
		$text = str_replace("'", "\'", $text);
		$text = str_replace("\n", "\\n", $text);
		$text = str_replace("\r", "\\r", $text);
		$text = str_replace("\t", "\\t", $text);
		return $text;
	}

	function stripsqlslashes($text) {
		$text = str_replace("\\n", "\n", $text);
		$text = str_replace("\\r", "\r", $text);
		$text = str_replace("\\t", "\t", $text);
		$text = str_replace("\'", "'", $text);
		$text = str_replace("\\\\", "\\", $text);
		return $text;
	}

	function sql_value_format(&$v) {
		$v = "'".addsqlslashes($v)."'";
	}

	define ("IN-GEKKO", true);

	require_once "../../conf.php";
	
	require_once GEKKO_ROOT."lib/auth.inc.php";

	varImport("action|", "id|", "return|");

	varRequire("action");

	if (is_admin()) {
	
		switch ($action) {
	
			case "create":
	
				varImport("file|", "opt", "compression|");
	
				// is filename specified? if not creating a generic name
				if (!$file)
					$file = preg_replace("/[^a-zA-Z0-9]/", "-", strtolower(_l("C_SITE.NAME")))."_backup_".date("YmdHis")."";
	
				// compression method
				switch ($compression) {
					// those are valid compression methods
					case "bz2": case "gzip": case "sql":
					break;
					default:
						// there was no compression method specified
						// checking for the best compression engine available
						if (extension_loaded("bz2")) {
							// bzip2 has a better compression ratio than zlib
							$compression = "bz2";
						} elseif (extension_loaded("zlib")) {
							$compression = "gzip";
						} else {
							$compression = "sql";
						}
					break;
				}

				$start_time = getmicrotime();

				// will save sql instructions
				$buff = "";
				
				// option: drop database
				if (isset($opt["drop_db"])) {
					$buff .= "# Dropping database \"".$db->info("name")."\"\n";
					$buff .= "DROP DATABASE IF EXISTS `".$db->info("name")."`;\n";
					$buff .= "CREATE DATABASE `".$db->info("name")."`;\n";
					$buff .= "USE `".$db->info("name")."`;\n\n";
				}
	
				// getting table names
				$tables = $db->get_tables();
	
				while ($table = $db->fetch_row($tables, SQL_NUM)) {
	
					$table = $table[0];
	
					// backup for tables using prefix
					if (($opt["table_prefix"]) && (substr($table, 0, strlen($opt["table_prefix"])) != $opt["table_prefix"]))
						continue;
	
					$buff .= "--\n";
					$buff .= "-- Table structure for table `$table`\n";
					$buff .= "--\n\n";

					// option: add drop table
					if (isset($opt["drop_table"])) {
						$buff .= "DROP TABLE IF EXISTS `$table`;\n";
					}

					// how this table was created?
					$q = $db->query("SHOW CREATE TABLE `$table`", __FILE__, __LINE__, true);
					$r = $db->fetch_row($q, SQL_NUM);
					$buff .= $r[1].";\n\n";

					// dumping table data
					$fields = "";
					$q = $db->query("SELECT * FROM `$table`", __FILE__, __LINE__, true);
					if ($db->num_rows($q)) {
						$buff .= "--\n";
						$buff .= "-- Dumping data for table `$table`\n";
						$buff .= "--\n\n";
						// listing table fields
						$fields = $db->get_fields($q);
						while ($row = $db->fetch_row($q, SQL_NUM)) {
							array_walk($row, "sql_value_format");
							$buff .= "INSERT INTO `{$table}` "."(".implode(", ", $fields).")"." VALUES (".implode(", ", $row).");\n";
						}
					}
	
					$buff .= "\n\n";
				}

				// backup header
				$header = "";
				$header .= "--\n";
				$header .= "-- Gekko ".GEKKO_VERSION." Database Backup (".$db->info("type").")\n";
				$header .= "-- Database Name: ".$db->info("name")."\n";
				$header .= "-- Date: ".date("r")."\n";
				$header .= "-- Host: ".$_SERVER["HTTP_HOST"]."\n";
				$header .= "-- Generation time: ".substr(getmicrotime() - $start_time, 0, 5)." seconds\n";
				$header .= "--\n\n";
	
				$file .= ".sql";
	
				// compressing backup data
				switch ($compression) {
					case "bz2":
						header("Content-disposition: filename=".$file.".bz2");
						header("Content-type: application/x-bzip2");
						$buff = bzcompress($buff);
					break;
					case "gzip":
						header("Content-disposition: filename=".$file.".gz");
						header("Content-type: application/x-gzip");
						$buff = gzencode($buff);
					break;
					case "sql":
						header("Content-disposition: filename=".$file);
						header("Content-type: text/plain");
					break;
				}
	
				header("Content-Length: ".strlen($buff)."");
				header("Pragma: no_cache");
				header("Expires: 0");
	
				// output buffer
				echo $buff;
	
				slog("backup: backup created", "actions", 1);
	
				die;
			break;
			case "restore":

				LoadLib (
					"file.lib.php"
				);

				varImport("file", "compression|");

				$start_time = getmicrotime();
	
				// getting submitted file
				$backup_file = file_get("file", "", "temp");
	
				// no filetype was specified
				if (!$compression) {
				
					// guessing filetype by identifing header (snippet borrowed from phpMyAdmin)
					$fh = fopen($backup_file["abs_loc"], "r");
					$hd = fread($fh, 3);
					fclose($fh);
	
					if ($hd == "BZh") {
						$compression = "bzip2";
					} elseif ($hd[0] == chr(31) && $hd[1] == chr(139)) {
						$compression = "gzip";
					}
				}
	
				// reading file into buffer
				$buff = "";
				switch ($compression) {
					case "bzip2":
						$fh = fopen($backup_file["abs_loc"], "r");
						$buff = bzdecompress(fread($fh, filesize($backup_file["abs_loc"])));
						fclose($fh);
					break;
					case "gzip":
						$fh = gzopen($backup_file["abs_loc"], "rb");
						while (!gzeof($fh)) {
							$buff .= gzgetc($fh);
						}
						gzclose($fh);
					break;
					default:
						$fh = fopen($backup_file["abs_loc"], "r");
						$buff = fread($fh, filesize($backup_file["abs_loc"]));
						fclose($fh);
					break;
				}
	
				// deleting backup file
				unlink($backup_file["abs_loc"]);
	
				$buff .= trim($buff)."\n";
	
				$i = 0;
				// parseing queries
				while ($buff) {
					if (($eol = strpos($buff, ";\n")) !== false) {
						// getting line
						$query = substr($buff, 0, $eol+2);
						// stripping line from buffer
						$buff = substr($buff, $eol+2);
					} else {
						// this seems to be EOF
						$query = $buff;
						$buff = "";
					}
					// removing commets and blank lines from query
					$check = explode("\n", $query);
					// setting query as null again
					$query = "";
					foreach ($check as $c) {
						// ignoring comments (lines that begin with "--" or "#") and blank lines
						if ($c && substr($c, 0, 1) != "#" && substr($c, 0, 2) != "--") {
							// appending this line to query (in case of multiple line queries)
							$query .= $c;
						}
					}
					if ($query) {
						// executing query
						$db->query($query, __FILE__, __LINE__);
						$i++;
					}
				}
	
				$time_taken = substr(getmicrotime() - $start_time, 0, 2);

				$return = urlEvalPrototype("index.php/module=admin/action=backup?message=".urlencode(sprintf(_l("L_DB_RESTORATION_SUCCESS"), $i, $time_taken)), true, false);

			break;
			default:
				abort(_l("L_UNDEFINED_ACTION"));
			break;
		}
	}

	redirect($return);

	db_exit();
?>

==> backup-0.7.module.tgz: modules/backup/main.act.php <==
<?php

	/*
	*	Gekko - Open Source Web Development Framework
	*	------------------------------------------------------------------------
	*	Copyright (C) 2004-2006, J. Carlos Nieto <xiam@users.sourceforge.net>
	*	This program is Free Software.
	*
	*	@package	Gekko
	*	@license	http://www.gnu.org/copyleft/gpl.html GNU/GPL License 2.0
	*	@author		J. Carlos Nieto <xiam@users.sourceforge.net>
	*	@link		http://www.gekkoware.org
	*/

	function addsqlslashes($text) {
		$text = str_replace("\\", "\\\\", $text);
		$text = str_replace("'", "\'", $text);
		$text = str_replace("\n", "\\n", $text);
		$text = str_replace("\r", "\\r", $text);
		$text = str_replace("\t", "\\t", $text);
		return $text;
	}

	function stripsqlslashes($text) {
		$text = str_replace("\\n", "\n", $text);
		$text = str_replace("\\r", "\r", $text);
		$text = str_replace("\\t", "\t", $text);
		$text = str_replace("\'", "'", $text);
		$text = str_replace("\\\\", "\\", $text);
		return $text;
	}

	function sql_value_format(&$v) {
		$v = "'".addsqlslashes($v)."'";
	}

	define ("IN-GEKKO", true);

	require_once "../../conf.php";

	require_once GEKKO_SOURCE_DIR."lib/auth.inc.php";

	varImport("action", "id", "return");

	varRequire("action");

	if (is_admin()) {

		switch ($action) {

			case "create":

				varImport("file", "opt", "compression");

				// is filename specified? if not creating a generic name
				if (!$file)
					$file = preg_replace("/[^a-zA-Z0-9]/", "-", strtolower(_l("C_SITE.NAME")))."_backup_".date("YmdHis")."";

				// compression method
				switch ($compression) {
					// those are valid compression methods
					case "bz2": case "gzip": case "sql":
					break;
					default:
						// there was no compression method specified
						// checking for the best compression engine available
						if (extension_loaded("bz2")) {
							// bzip2 has a better compression ratio than zlib
							$compression = "bz2";
						} elseif (extension_loaded("zlib")) {
							$compression = "gzip";
						} else {
							$compression = "sql";
						}
					break;
				}

				$start_time = getmicrotime();

				// will save sql instructions
				$buff = "";

				// option: drop database
				if (isset($opt["drop_db"])) {
					$buff .= "# Dropping database \"".$db->info("name")."\"\n";
					$buff .= "DROP DATABASE IF EXISTS `".$db->info("name")."`;\n";
					$buff .= "CREATE DATABASE `".$db->info("name")."`;\n";
					$buff .= "USE `".$db->info("name")."`;\n\n";
				}

				// getting table names
				$tables = $db->getTables();

				while ($table = $db->fetchRow($tables, SQL_NUM)) {

					$table = $table[0];

					// backup for tables using prefix
					if (($opt["table_prefix"]) && (substr($table, 0, strlen($opt["table_prefix"])) != $opt["table_prefix"]))
						continue;

					$buff .= "--\n";
					$buff .= "-- Table structure for table `$table`\n";
					$buff .= "--\n\n";

					// option: add drop table
					if (isset($opt["drop_table"])) {
						$buff .= "DROP TABLE IF EXISTS `$table`;\n";
					}

					// how this table was created?
					$q = $db->query("SHOW CREATE TABLE `$table`", __FILE__, __LINE__, true);
					$r = $db->fetchRow($q, SQL_NUM);
					$buff .= $r[1].";\n\n";

					// dumping table data
					$fields = "";
					$q = $db->query("SELECT * FROM `$table`", __FILE__, __LINE__, true);
					if ($db->numRows($q)) {
						$buff .= "--\n";
						$buff .= "-- Dumping data for table `$table`\n";
						$buff .= "--\n\n";
						// listing table fields
						$fields = $db->getFields($q);
						while ($row = $db->fetchRow($q, SQL_NUM)) {
							array_walk($row, "sql_value_format");
							$buff .= "INSERT INTO `{$table}` "."(".implode(", ", $fields).")"." VALUES (".implode(", ", $row).");\n";
						}
					}

					$buff .= "\n\n";
				}

				// backup header
				$header = "";
				$header .= "--\n";
				$header .= "-- Gekko ".GEKKO_VERSION." Database Backup (".$db->info("type").")\n";
				$header .= "-- Database Name: ".$db->info("name")."\n";
				$header .= "-- Date: ".date("r")."\n";
				$header .= "-- Host: ".$_SERVER["HTTP_HOST"]."\n";
				$header .= "-- Generation time: ".substr(getmicrotime() - $start_time, 0, 5)." seconds\n";
				$header .= "--\n\n";

				$file .= ".sql";

				// compressing backup data
				switch ($compression) {
					case "bz2":
						header("Content-disposition: filename=".$file.".bz2");
						header("Content-type: application/x-bzip2");
						$buff = bzcompress($buff);
					break;
					case "gzip":
						header("Content-disposition: filename=".$file.".gz");
						header("Content-type: application/x-gzip");
						$buff = gzencode($buff);
					break;
					case "sql":
						header("Content-disposition: filename=".$file);
						header("Content-type: text/plain");
					break;
				}

				header("Content-Length: ".strlen($buff)."");
				header("Pragma: no_cache");
				header("Expires: 0");

				// output buffer
				echo $buff;

				slog("backup: backup created", "actions", 1);

				die;
			break;
			case "restore":

				LoadLib (
					"file.lib.php"
				);

				varImport("file", "compression");

				$start_time = getmicrotime();

				// getting submitted file
				$backup_file = file_get("file", "", "temp");

				// no filetype was specified
				if (!$compression) {

					// guessing filetype by identifing header (snippet borrowed from phpMyAdmin)
					$fh = fopen($backup_file["abs_loc"], "r");
					$hd = fread($fh, 3);
					fclose($fh);

					if ($hd == "BZh") {
						$compression = "bzip2";
					} elseif ($hd[0] == chr(31) && $hd[1] == chr(139)) {
						$compression = "gzip";
					}
				}

				// reading file into buffer
				$buff = "";
				switch ($compression) {
					case "bzip2":
						$fh = fopen($backup_file["abs_loc"], "r");
						$buff = bzdecompress(fread($fh, filesize($backup_file["abs_loc"])));
						fclose($fh);
					break;
					case "gzip":
						$fh = gzopen($backup_file["abs_loc"], "rb");
						while (!gzeof($fh)) {
							$buff .= gzgetc($fh);
						}
						gzclose($fh);
					break;
					default:
						$fh = fopen($backup_file["abs_loc"], "r");
						$buff = fread($fh, filesize($backup_file["abs_loc"]));
						fclose($fh);
					break;
				}

				// deleting backup file
				unlink($backup_file["abs_loc"]);

				$buff .= trim($buff)."\n";

				$i = 0;
				// parseing queries
				while ($buff) {
					if (($eol = strpos($buff, ";\n")) !== false) {
						// getting line
						$query = substr($buff, 0, $eol+2);
						// stripping line from buffer
						$buff = substr($buff, $eol+2);
					} else {
						// this seems to be EOF
						$query = $buff;
						$buff = "";
					}
					// removing commets and blank lines from query
					$check = explode("\n", $query);
					// setting query as null again
					$query = "";
					foreach ($check as $c) {
						// ignoring comments (lines that begin with "--" or "#") and blank lines
						if ($c && substr($c, 0, 1) != "#" && substr($c, 0, 2) != "--") {
							// appending this line to query (in case of multiple line queries)
							$query .= $c;
						}
					}
					if ($query) {
						// executing query
						$db->query($query, __FILE__, __LINE__);
						$i++;
					}
				}

				$time_taken = substr(getmicrotime() - $start_time, 0, 2);

				$return = urlEvalPrototype("index.php/module=admin/action=backup?message=".urlencode(sprintf(_l("L_DATABASE_RESTORED"), $i, $time_taken)), true, false);

			break;
			default:
				abort(_l("L_UNDEFINED_ACTION"));
			break;
		}
	}

	redirect($return);

	db_exit();
?>
